/* Nessus
 * Copyright (C) 1999 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
#include <includes.h>
#ifdef USE_GTK
#include <gtk/gtk.h>
#include "error_dialog.h"
#include "password_dialog.h"
#include "xstuff.h"
static void pass_build_dialog(struct arglist *, char *);
static void pass_dialog_callback(void *, struct arglist *);
#endif
#include "globals.h"

char * pass_dialog();

#ifdef NESSUSNT
/*
 * Quick'n'dirty replacement for getpass(3)
 * since NT lacks it. 
 * 
 * TODO : turn the echo off when the use types his
 *        password
 */
static char *
getpass(str)
   char * str;
{
  static char pass[255];
  printf("%s", str);
  bzero(pass, 255);
  fgets(pass, 254, stdin);
  return pass;
}
#endif

#ifdef USE_GTK

static void
pass_build_dialog(ctrls, prompt)
 struct arglist * ctrls;
 char * prompt;
{
 GtkWidget * w, * table, * entry, * ok, * frame;
 
 
 w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 gtk_window_position(GTK_WINDOW(w), GTK_WIN_POS_CENTER);
 gtk_window_set_title(GTK_WINDOW(w), "Account password");
 gtk_container_border_width(GTK_CONTAINER(w), 5);
 arg_add_value(ctrls, "WINDOW", ARG_PTR, -1, w);
 
 frame = gtk_frame_new(prompt);
 gtk_container_add(GTK_CONTAINER(w), frame);
 gtk_widget_show(frame);
 
 table  = gtk_table_new(2,2,TRUE);
 gtk_container_border_width(GTK_CONTAINER(frame), 5);
 gtk_container_add(GTK_CONTAINER(frame), table);
 gtk_container_border_width(GTK_CONTAINER(table), 10);
 gtk_widget_show(table);
 
 
 entry = gtk_entry_new();
 gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 gtk_table_attach_defaults(GTK_TABLE(table), entry,0,2,0,1);
 gtk_widget_show(entry); 
 arg_add_value(ctrls, "PASSWORD", ARG_PTR, -1, entry);
 
 ok = gtk_button_new_with_label("Ok");
 GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
 gtk_signal_connect(GTK_OBJECT(ok), "clicked", 
 		    (GtkSignalFunc)pass_dialog_callback, 
		    (void*)ctrls);
		    
 gtk_table_attach_defaults(GTK_TABLE(table), ok, 1,2,1,2);
 gtk_widget_show(ok);
 gtk_widget_show(w);
 gtk_grab_add(w);
}


static void pass_dialog_callback(nul, ctrls)
 void * nul;
 struct arglist * ctrls;
{
 GtkWidget * pass = (GtkWidget *)arg_get_value(ctrls, "PASSWORD");
 char * pass_str;

 gtk_grab_remove(arg_get_value(ctrls, "WINDOW"));
 pass_str = gtk_entry_get_text(GTK_ENTRY(pass));
 if(!pass_str || !strlen(pass_str))
 	show_error("You must enter a valid password");
 else {
    	char * s = estrdup(pass_str);
        close_window(NULL, arg_get_value(ctrls, "WINDOW"));
 	arg_add_value(ctrls, "PASSWORD_TEXT", ARG_STRING, strlen(s), s);
       }
}


char *
pass_dialog (int unused)
{
 struct arglist * ctrls;
 char * ret;
 
 ctrls = emalloc(sizeof(struct arglist));
 pass_build_dialog(ctrls, "Password required : ");
 while(!arg_get_value(ctrls, "PASSWORD_TEXT"))
  while(gtk_events_pending())gtk_main_iteration();
 ret = estrdup(arg_get_value(ctrls, "PASSWORD_TEXT"));
 arg_free(ctrls);
 return(ret);
}

char *
keypass_dialog (int unused)
{
 struct arglist * ctrls;
 char * ret;
 
 ctrls = emalloc(sizeof(struct arglist));
 pass_build_dialog(ctrls, "Pass phrase : ");
 while(!arg_get_value(ctrls, "PASSWORD_TEXT"))
  while(gtk_events_pending())gtk_main_iteration();
 ret = estrdup(arg_get_value(ctrls, "PASSWORD_TEXT"));
 arg_free(ctrls);
 return(ret);
}
#endif /* USE_GTK */

/* non GUI password dialogue */
char* 
cmdline_pass
  (int unused)
{
  return getpass ("Server password: "); 
}

 
/* used for private key activation */
static int __created_key = 0;

int /* return just-generated-private-key status */
created_private_key 
 (void)
{
  int n = __created_key ;
  __created_key = 0 ;
  return n;
}

char* 
get_pwd 
 (int mode) {
  char *s, *t ;

  switch (mode) {
  case 0:
    /* key activation mode */
    if(F_quiet_mode)return getpass("Pass phrase: ");
    else
#ifdef USE_GTK
    return keypass_dialog (0);
#else
    return getpass("Pass phrase: ");
#endif    
  case 2:
    __created_key ++ ;
#   ifdef FIRST_PWD_BLURB
    fflush (stderr);
    printf (FIRST_PWD_BLURB);
    fflush (stdout);
#   endif
  }

  if ((s = getpass ("New pass phrase: ")) == 0)
    return (char*)-1;
  s = estrdup (s);
  if ((t = getpass ("Repeat         : ")) == 0) {
    efree (&s);
    return (char*)-1;
  }
  if (strcmp (s, t) != 0)
    t = 0 ;
  efree (&s);
  if (t == 0)
    return (char*)-1;
  return t ;
}
 
 
 
